import { ChildProcessWithoutNullStreams, spawn } from "child_process";
import Emittery from "emittery";

export default function simpleSpawn(
  cmd: string,
  arg: string[]
): Promise<string> {
  const process = spawn(cmd, arg);
  let output = "";
  let err = "";
  return new Promise((resolve, reject) => {
    process.on("close", (code) => {
      code === 0 ? resolve(output) : reject(err);
    });
    process.stdout.on("data", (data) => (output += data.toString("utf-8")));
    process.stderr.on("data", (data) => (err += data.toString("utf-8")));
  });
}

class DeferedSpawn extends Emittery {
  constructor(private process: ChildProcessWithoutNullStreams) {
    super();
  }

  async waitForAnswer() {
    return new Promise((resolve, reject) => {
      this.on("success", resolve);
      this.on("error", reject);
    });
  }

  async waitForNextData(): Promise<string | null> {
    return new Promise((resolve, reject) => {
      this.on("data", (data) => resolve(data.toString("utf-8")));
      this.on("error", reject);
      this.on("end", () => resolve(null));
    });
  }

  kill() {
    process.kill(-this.process.pid); // https://stackoverflow.com/a/49842576
    debugger;
    console.log("killed map!");
  }
}

export function deferedSpawn(
  cmd: string,
  arg: string[]
): Promise<DeferedSpawn> {
  const process = spawn(cmd, arg, { detached: true });
  let spawned = false;
  let output = "";
  let err = "";
  const emitter = new DeferedSpawn(process);
  return new Promise((resolve, reject) => {
    process.stdout.on("data", (data) => {
      output += data.toString("utf-8");
      emitter.emit("data", data);
    });
    process.stderr.on("data", (data) => {
      err += data.toString("utf-8");
    });
    process.on("spawn", () => {
      resolve(emitter);
    });
    process.on("close", (code) => {
      if (!spawned) {
        reject(err);
      } else {
        if (code === 0) {
          emitter.emit("success", output);
        } else {
          emitter.emit("error", err);
        }
        emitter.emit("end", { err, output });
      }
    });
  });
}
